#!/usr/bin/perl
#$Id: RO_GPS_client_term.pl,v 1.1 2002/10/23 07:14:13 white Exp $
use Socket;
use IO::Select;
use IO::Socket;
use RO_GPS;
use POSIX qw(:termios_h);
use Term::Cap;
require '../jcode.pl';

if(@ARGV != 2){
    print STDERR "usage: $0 server ID\n";
    exit(1);
}else{
    $serv = shift;
    $ID   = shift;
}

my $port = 5963;
my @dummy =
(&RO_GPS::version(),$ID,"----","----","",0,0,0,0,"","Ż");
map{&jcode::convert(\$_,'sjis');} @dummy;
my %dummy;
@dummy{"APPRI_VER","ID","serv","name","place","HP","MaxHP","SP","MaxSP","Lv","Mes"} = @dummy;

# termios for stdin
my $termios = new POSIX::Termios;
$termios->getattr( fileno(STDIN) );
$lflag = $termios->getlflag();
$termios->setlflag($lflag & ~(ECHO | ECHOK | ICANON));
$termios->setattr( fileno(STDIN), TCSANOW );


# termcap for stdout
my $ospeed  = $termios->getospeed;
my $terminal = Tgetent Term::Cap { TERM => undef , OSPEED => $ospeed };
$terminal->Trequire(qw/ce ku kd/);

$terminal->Tgoto('cm',0,0,STDOUT);
$terminal->Tputs('cl', $count = 1, STDOUT);

print &RO_GPS::client_Labels();

# connect
$selecter = IO::Select->new;
$server   = IO::Socket::INET->new(PeerHost  => $serv,
				  PeerPort  => $port,
				  Proto     => 'tcp' );
$server->autoflush(1);

$stdin    = IO::Handle->new_from_fd(STDIN,"r");
$stdout   = IO::Handle->new_from_fd(STDOUT,"w");
$stdout->autoflush(1);

$selecter->add($server);
$selecter->add($stdin);



$server->printflush(&RO_GPS::make_client_msg(%dummy));
$timeout = 1;
my $buf_mes,@buf_mes;
while(1){
    my $io;
    my(@io) = $selecter->can_read($timeout);
    if(scalar(@io) == 0){
	$server->printflush(&RO_GPS::make_client_msg(%dummy));
    }
    foreach $io(@io){
	my $buf;
	if($io == $server){
	    $io->sysread($buf,8192);
	    my @clients = &RO_GPS::parse_serv_msg($buf);
	    $terminal->Tgoto('cm',0,1,STDOUT);
	    foreach my $client (@clients){
		$terminal->Tputs('dl', $count = 1, STDOUT);
		my $line = &RO_GPS::client_toString(%$client);
		&jcode::convert(\$line,'euc');
		print $line;
	    }
	    $terminal->Tputs('dl', $count = 1, STDOUT);
	    $stdout->printflush($buf_mes);
	    
	    print SERV &RO_GPS::make_client_msg(%dummy);
	}elsif($io == $stdin){
	    my $c;
	    while($stdin->read($c,1) && $c){
		if($c eq "\n" || $c eq "\r"){
		    $buf_mes = join('',@buf_mes);
		    &jcode::convert(\$buf_mes,'sjis');
		    $dummy{"Mes"} = $buf_mes;
		    $buf_mes = "";
		    undef(@buf_mes);
		    last;
		}elsif($c eq "\010" || $c eq "\177"){
		    pop(@buf_mes);
		    $buf_mes = join('',@buf_mes);
		    $terminal->Tputs('dl', 1, STDOUT);
		    $stdout->printflush("\r".$buf_mes);
		    last;
		}else{
		    $stdin->read($c,1,1) if($c =~ /[\xA1-\xFE]/);
		    push(@buf_mes,$c);
		    $buf_mes = join('',@buf_mes);
		    $stdout->printflush($c);
		}
	    }
	}
    }
}

while(<SERV>){
}
print "\n--connection closed\n";

END{
    if($termios){
	$termios->setlflag($lflag);
	$termios->setcc(VTIME, 0);
	$termios->setattr( fileno(STDIN), TCSANOW );
    }
}
